﻿/*
 * by Björn Hampe.  http://www.visualgaze.de/software.htm
 * v1.1 2009/03/11
 *
 */

/** "$$$/JavaScripts/ResizeOnOpen/Description=Pop the image size dialog." **/

// enable double clicking from the Macintosh Finder or the Windows Explorer
#target photoshop

// in case we double clicked the file
app.bringToFront();

// on localized builds we pull the $$$/Strings from a .dat file, see documentation for more details
$.localize = true;
// UI strings to be localized
var strTitle = localize("$$$/JavaScripts/Scale/Title=Scale to defined minimum dimensions");
var strLblWidth = localize("$$$/JavaScripts/Scale/Width=min width");
var strLblHeight = localize("$$$/JavaScripts/Scale/Height=min height");
var strLblFillValues = localize("$$$/JavaScripts/Scale/FillValues=Fill in the min width and min height");
var strLblSharpen = localize("$$$/JavaScripts/Scale/Sharpen=Unsharp sharpen the image");
var strButtonCancel = localize("$$$/JavaScripts/Scale/Cancel=Cancel");
var strButtonRun = localize("$$$/JavaScripts/Scale/Run=Run");
var strAlertDocumentMustBeOpened = localize("$$$/JavaScripts/Scale/OneDocument=You must have a document open to scale!");
var strAlertSpecifyWidth = localize("$$$/JavaScripts/Scale/SpecifyWidth=Please specify width.");
var strAlertSpecifyHeight = localize("$$$/JavaScripts/Scale/SpecifyHeight=Please specify height.");
var strAlertDocDimsToLow = localize("$$$/JavaScripts/Scale/DocDimsToLow=Document dimensions too low. Scale only if document dimensions higher than minimum scales.");

var sConfigFile = 'skalieren.cfg';
var defaultMinWidth = 800;
var defaultMinHeight = 600;
var iSharpenStage1 = 200;
var iSharpenStage2 = 800;
var iSharpenStage3 = 1500;

var minMultiplier = defaultMinWidth/defaultMinHeight;
var bSharpen = true;

var iConfigLength = 3;

main();

///////////////////////////////////////////////////////////////////////////////
// Function: settingDialog
// Usage: create the main dialog
// Input: settings to initialize the dialog with , ScaleInfo object
// Return: true if ok, false if cancel
///////////////////////////////////////////////////////////////////////////////
function settingDialog(scaleInfo)  {
	dlgMain = new Window("dialog", strTitle);
	dlgMain.orientation = 'column';
	dlgMain.alignChildren = 'left';

	// -- top of the dialog, first line
	dlgMain.add("statictext", undefined, strLblFillValues); // Label: Eingabe tätigen

	// -- two groups, one for left and one for right ok, cancel
	dlgMain.grp1 = dlgMain.add("group");
	dlgMain.grp1.orientation = 'row';
	dlgMain.grp1.alignChildren = 'top';
	dlgMain.grp1.alignment = 'fill';

	// -- group contains four lines
	dlgMain.grp1Left = dlgMain.grp1.add("group");
	dlgMain.grp1Left.orientation = 'column';
	dlgMain.grp1Left.alignChildren = 'left';
	dlgMain.grp1Left.alignment = 'fill';
	// -- the second line in the dialog
	dlgMain.grp2Line = dlgMain.grp1Left.add("group");
	dlgMain.grp2Line.orientation = 'row';
	dlgMain.grp2Line.alignChildren = 'center';
	// -- the third line in the dialog
	dlgMain.grp3Line = dlgMain.grp1Left.add("group");
	dlgMain.grp3Line.orientation = 'row';
	dlgMain.grp3Line.alignChildren = 'fill';
	// -- the fourth line in the dialog
	dlgMain.grp4Line = dlgMain.grp1Left.add("group");
	dlgMain.grp4Line.orientation = 'row';
	dlgMain.grp4Line.alignChildren = 'fill';

	// input fields for width and height
  dlgMain.grp2Line.add("statictext", undefined, strLblWidth); // Label Breite
	dlgMain.inpWidth = dlgMain.grp2Line.add("edittext", undefined, scaleInfo.minWidth.toString());
	dlgMain.inpWidth.alignment = 'fill';
	dlgMain.inpWidth.preferredSize.width = 40;
  dlgMain.grp2Line.add("statictext", undefined, strLblHeight); // Label Höhe
	dlgMain.inpHeight = dlgMain.grp2Line.add("edittext", undefined, scaleInfo.minHeight.toString());
	dlgMain.inpHeight.preferredSize.width = 40;

	// sharpen
	dlgMain.chkSharpen = dlgMain.grp3Line.add( 'checkbox', undefined, strLblSharpen ); // Label sharpen
	dlgMain.chkSharpen.value = scaleInfo.bSharpen;

	// Buttons
	dlgMain.btnRun = dlgMain.grp4Line.add("button", undefined, strButtonRun );
  dlgMain.btnRun.onClick = btnRunOnClick;

	dlgMain.btnCancel = dlgMain.grp4Line.add("button", undefined, strButtonCancel );
  dlgMain.btnCancel.onClick = function() {
		var d = this;
		while (d.type != 'dialog') {
			d = d.parent;
		}
		d.close(false);
	}

	dlgMain.defaultElement = dlgMain.btnRun;
	dlgMain.cancelElement = dlgMain.btnCancel;

	dlgMain.center();
	var result = dlgMain.show();
	if ( false == result) {
		return result;
	}

	// get setting from dialog
	scaleInfo.minWidth  = dlgMain.inpWidth.text;
	scaleInfo.minHeight = dlgMain.inpHeight.text;
	scaleInfo.bSharpen = dlgMain.chkSharpen.value;

  saveLastDims(scaleInfo.minWidth,scaleInfo.minHeight,scaleInfo.bSharpen);

	return result;
}

///////////////////////////////////////////////////////////////////////////////
// Function: btnRunOnClick
// Usage: routine is assigned to the onClick method of the run button
// Input: checks the dialog params and closes with the dialog with true
// Return: <none>, dialog is closed with true on success
///////////////////////////////////////////////////////////////////////////////
function btnRunOnClick() {
	// check if the setting is properly
	if (dlgMain.inpWidth.text == '') {
		alert(strAlertSpecifyWidth);
		return;
	}
	if (dlgMain.inpHeight.text == '') {
		alert(strAlertSpecifyHeight);
		return;
	}
	if (dlgMain.inpWidth.text > app.activeDocument.width || dlgMain.inpHeight.text > app.activeDocument.height) {
		alert(strAlertDocDimsToLow);
		return;
	}

	// find the dialog in this auto layout mess
	var d = this;
	while (d.type != 'dialog') {
		d = d.parent;
	}
	d.close(true);
}

///////////////////////////////////////////////////////////////////////////////
// Function: ScaleInfo
// Usage: object for holding the dialog parameters
// Input: <none>
// Return: object holding the scale info
///////////////////////////////////////////////////////////////////////////////
function ScaleInfo() {
		var aDims = loadLastDims();
		if (aDims) {
	    this.minWidth = parseInt(aDims[0]);
	    this.minHeight = parseInt(aDims[1]);
	    this.bSharpen = aDims[2];
		} else {
	    this.minWidth = defaultMinWidth;
	    this.minHeight = defaultMinHeight;
	    this.bSharpen = bSharpen;
	  }
}

///////////////////////////////////////////////////////////////////////////////
// Function: loadLastDims
// Usage: load the last used dimensions
// Input: <none>
// Return: dimensions
///////////////////////////////////////////////////////////////////////////////
// siehe Bildprozessor.jsx
function loadLastDims() {
	Folder.current = Folder("~/");
	var oFile = new File(sConfigFile);
	if (oFile.exists) {
		oFile.open( "r" );
		var config = oFile.read();
		oFile.close();
		var configDims = config.split('|');
		if (configDims.length != iConfigLength) {
			return false;
		} else {
			return configDims;
		}
	} else {
	  return false;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Function: saveLastDims
// Usage: save the last used dimensions
// Input: <none>
// Return: <none>
///////////////////////////////////////////////////////////////////////////////
// siehe Bildprozessor.jsx
function saveLastDims(width,height,sharpen) {
	Folder.current = Folder( "~/" );
	var oFile = new File(sConfigFile);
	oFile.open( "w", "TEXT", "????" );
	oFile.writeln(width+"|"+height+"|"+sharpen);
	oFile.close();
}

///////////////////////////////////////////////////////////////////////////////
// Function: main
// Usage: the main routine for this JavaScript
// Input: <none>
// Return: <none>
///////////////////////////////////////////////////////////////////////////////
function main() {
  if ( app.documents.length <= 0 ) {
  	alert(strAlertDocumentMustBeOpened);
  	return;
  }
	var scaleInfo = new ScaleInfo();
	if (false == settingDialog(scaleInfo))  {
		return; // quit
	}
	try {
		// Save the current preferences
		var startRulerUnits = app.preferences.rulerUnits;
		var startTypeUnits = app.preferences.typeUnits;
		// Set Adobe Photoshop CS2 to use pixels
		app.preferences.rulerUnits = Units.PIXELS;
		app.preferences.typeUnits = TypeUnits.PIXELS;

	  var docRef = app.activeDocument;
	  var docWidth = docRef.width.as("px");
	  var docHeight = docRef.height.as("px");
	  var currentMultiplier = docWidth/docHeight;
	  var destMultiplier = scaleInfo.minWidth/scaleInfo.minHeight;

		// change the color mode to RGB.  Important for resizing GIFs with indexed colors, to get better results
		docRef.changeMode(ChangeMode.RGB);

		if (currentMultiplier > destMultiplier) {
	    // Multiplier größer minMultiplier
	    var iBreite = docWidth/docHeight*scaleInfo.minHeight;
	    var iHoehe = scaleInfo.minHeight;
			displayDialogs = DialogModes.NO; // NO = auto, without dialog; ALL = show dialog
			docRef.resizeImage( +iBreite, +iHoehe, 72, ResampleMethod.BICUBIC );
			displayDialogs = DialogModes.NO;
	  } else {
	    // Multiplier kleiner minMultiplier
	    var iBreite = scaleInfo.minWidth;
	    var iHoehe = docHeight*scaleInfo.minWidth/docWidth;
			displayDialogs = DialogModes.NO; // NO = auto, without dialog; ALL = show dialog
			docRef.resizeImage( +iBreite, +iHoehe, 72, ResampleMethod.BICUBIC );
			displayDialogs = DialogModes.NO;
	  }
	  //docRef.flatten;
		if (docRef.layers.length > 1) docRef.mergeVisibleLayers();
		if (scaleInfo.bSharpen) {
			if (docRef.width.value <= iSharpenStage1) {
			  // bis 200er Auflösung
				docRef.activeLayer.applyUnSharpMask(70, 0.5, 0);
		  } else if (docRef.width.value > iSharpenStage1 && docRef.width.value <= iSharpenStage2) {
			  // bis 800er Auflösung
			  docRef.activeLayer.applyUnSharpMask(85, 0.7, 0);
		  } else if (docRef.width.value > iSharpenStage2 && docRef.width.value <= iSharpenStage3) {
			  // bis 1500er Auflösung
			  docRef.activeLayer.applyUnSharpMask(100, 0.7, 0);
		  } else {
		    // ab 1500er Auflösung
			  docRef.activeLayer.applyUnSharpMask(100, 1.0, 0);
		  }
	  }
		selRegion = Array(
	     Array((parseInt(docRef.width.value)/2)-(parseInt(scaleInfo.minWidth)/2), (parseInt(docRef.height.value)/2)-(parseInt(scaleInfo.minHeight)/2)),
	     Array((parseInt(docRef.width.value)/2)-(parseInt(scaleInfo.minWidth)/2)+parseInt(scaleInfo.minWidth), (parseInt(docRef.height.value)/2)-(parseInt(scaleInfo.minHeight)/2)),
	     Array((parseInt(docRef.width.value)/2)-(parseInt(scaleInfo.minWidth)/2)+parseInt(scaleInfo.minWidth), (parseInt(docRef.height.value)/2)-(parseInt(scaleInfo.minHeight)/2)+parseInt(scaleInfo.minHeight)),
	     Array((parseInt(docRef.width.value)/2)-(parseInt(scaleInfo.minWidth)/2), (parseInt(docRef.height.value)/2)-(parseInt(scaleInfo.minHeight)/2)+parseInt(scaleInfo.minHeight))
	  )
	  //cropRegion = Array(0, 0, scaleInfo.minWidth, scaleInfo.minHeight)
	  docRef.selection.select(selRegion)

		docRef = null;

		// Reset the application preferences
		app.preferences.rulerUnits = startRulerUnits;
		app.preferences.typeUnits = startTypeUnits;
	}
	catch(e) {
		// always wrap your script with try/catch blocks so you don't stop production
		// remove comments below to see error for debugging
		alert( e );
	}
}

